// Two country model with incomplete financial markets,
// traded, and nontraded goods. 


var cc, ccn, cct, cch, ccm, le, len, let, 
    eta, lambda, gamman, gammat, gdp, yn, yt, 
    j, jn, jt, in, it, ih, im, kpn, kpt, 
    wn, wt, rn, rt, nn, nt, rs, pc, pct, 
    pj, pit, pn, pf, nfa, nfay, tby, q, sy, jy
    
    cc2, ccn2, cct2, cch2, ccm2, le2, len2, let2, 
    eta2, lambda2, gamman2, gammat2, gdp2, yn2, yt2, 
    j2, jn2, jt2, in2, it2, ih2, im2, kpn2, kpt2,
    wn2, wt2, rn2, rt2, nn2, nt2, rs2, pc2, pct2,
    pj2, pit2, pn2, pf2, tby2, q2, sy2, jy2;
    
varexo nnerr, nterr, nnerr2, nterr2;

// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// % define parameters here
// %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

parameters beta_p, chi_p, le_p, delta_p, alphan_p, alphat_p 
           chi0_p, rhonn_p, rhont_p, omegacn_p, omegact_p,
           omegain_p, omegait_p, rhon_p, rhot_p, nfa_p, psik_p;


// Discount factor  (default .99)
beta_p = 0.99;

// Frisch Elasticity of labor supply (2/chi_p);
chi_p = 10;

// Steady state labor supply
le_p = 0.5;

// depreciation rate (default 0.025)
delta_p = 0.025;
 
// Capital share
alphan_p = 0.3;
alphat_p = 0.3;

// consumption and investment weights
omegacn_p = 0.5;
omegact_p = 0.76;

omegain_p = 0.5;
omegait_p = 0.76;

// Rental rate for capital in both sectors
r_p = (delta_p - 1 + 1/beta_p);

// sectoral saving rate
sn_p = alphan_p * delta_p/r_p;
st_p = alphat_p * delta_p/r_p;

// relative size of sectors
yn2y_p = (omegacn_p*(1-st_p) + omegain_p*st_p) /(1-omegacn_p*(st_p - sn_p)-omegain_p*(sn_p-st_p));
yt2y_p = 1-yn2y_p;

// aggregate saving rate
s_p = yn2y_p*sn_p + yt2y_p*st_p;

// output
y_p = le_p/( yt2y_p * (delta_p/st_p)^(alphat_p/(1-alphat_p)) + yn2y_p * (delta_p/sn_p)^(alphan_p/(1-alphan_p)) );
yt_p = yt2y_p * y_p;
yn_p = yn2y_p * y_p;

// sectoral labor
let_p = yt2y_p*(delta_p/st_p)^(alphat_p/(1-alphat_p))*y_p;
len_p = yn2y_p*(delta_p/sn_p)^(alphan_p/(1-alphan_p))*y_p;

chi0_p = (1-le_p)^(chi_p)*(1/((1-s_p)*y_p)) *(1-alphat_p)*(yt_p/let_p);

// elasticity of substitution between domestic and foreign traded goods
el = 2;
rhot_p = 1/(el-1);

// elasticity of substitution between traded and nontraded goods
el = 0.5;
rhon_p = 1/(el-1);

// shock persistence parameters
rhonn_p = 0.95;    
rhont_p = 0.95;

// cost of adjustment for capital
psik_p = 0.5;
     
// parameter governing debt elasticity of interest rate
nfa_p = 0.01;
  
model;
///////////////////////////////////////////////////////////////////////////
// Country 1
///////////////////////////////////////////////////////////////////////////

// 1. FOC with respect to consumption
1/cc - lambda*pc=0;   

// 2. FOC wrt to aggregate labor
-chi0_p*(1-le)^(-chi_p) + eta =0;

// 3. FOC wrt to labor in each sector
lambda*pn*wn - eta = 0;
lambda*wt - eta = 0;

// 5. FOC with respect to investment
gamman- lambda*pj - gamman*psik_p*(jn/kpn(-1)-delta_p)= 0;
gammat- lambda*pj - gamman*psik_p*(jt/kpt(-1)-delta_p)= 0;


// 7. gamman shadow price of capital
(1-delta_p)*beta_p * gamman(1) - gamman + pn(1)*beta_p*lambda(1)*rn(1) + beta_p*gamman(1)*psik_p*(jn(1)/kpn-delta_p)/kpn = 0;
(1-delta_p)*beta_p * gammat(1) - gammat +       beta_p*lambda(1)*rt(1) + beta_p*gammat(1)*psik_p*(jt(1)/kpt-delta_p)/kpt= 0;

// 9. technology constraints
yn = kpn(-1)^alphan_p*(len*nn)^(1-alphan_p);
yt = kpt(-1)^alphat_p*(let*nt)^(1-alphat_p);

// 11. definition of aggregate output
gdp = pn*yn+yt;

// 12. resource constraints
yn = ccn + in;
yt = cch + ih + ccm2+im2;
j = jt + jn;
le = let + len;

// 16. definition of consumption aggregator 
cc  = (omegacn_p^(rhon_p/(1+rhon_p))*ccn^(1/(1+rhon_p))+(1-omegacn_p)^(rhon_p/(1+rhon_p))*cct^(1/(1+rhon_p)))^(1+rhon_p);
cct = (omegact_p^(rhot_p/(1+rhot_p))*cch^(1/(1+rhot_p))+(1-omegact_p)^(rhot_p/(1+rhot_p))*ccm^(1/(1+rhot_p)))^(1+rhot_p);

// 18. demand equations
ccn = omegacn_p*(pn/pc)^(-(1+rhon_p)/rhon_p) *cc;
cct = (1-omegacn_p)*(pct/pc)^(-(1+rhon_p)/rhon_p) *cc;
cch = omegact_p*(1/pct)^(-(1+rhot_p)/rhot_p) *cct;
ccm = (1-omegact_p)*(pf/pct)^(-(1+rhot_p)/rhot_p) *cct;


// 22. definition of consumption aggregator (pins down ccd)
j =  (omegain_p^(rhon_p/(1+rhon_p))*in^(1/(1+rhon_p))+(1-omegain_p)^(rhon_p/(1+rhon_p))*it^(1/(1+rhon_p)))^(1+rhon_p);
it = (omegait_p^(rhot_p/(1+rhot_p))*ih^(1/(1+rhot_p))+(1-omegait_p)^(rhot_p/(1+rhot_p))*im^(1/(1+rhot_p)))^(1+rhot_p);

// 24. demand equations
in = omegain_p*(pn/pj)^(-(1+rhon_p)/rhon_p) *j;
it = (1-omegain_p)*(pit/pj)^(-(1+rhon_p)/rhon_p) *j;
ih = omegait_p*(1/pit)^(-(1+rhot_p)/rhot_p) *it;
im = (1-omegait_p)*(pf/pit)^(-(1+rhot_p)/rhot_p) *it;


// 28. nfa
pf*nfa/(1+rs2)/exp(-nfa_p/2*(pf*nfa/gdp)) = pf*(nfa(-1)) + ccm2 + im2 -pf*ccm - pf*im ;

// 29. nfa to gdp
nfay = pf*nfa/gdp;

// 30. Trade Balance, share of GDP
tby = (ccm2+im2-pf*ccm-pf*im)/gdp;

// 31. Saving rate
sy = 1-cc/gdp;

// 32. Investment rate
jy = j/gdp;

// 33. UIP equation
1/(1+rs)*pf(1)/pf = 1/(1+rs2)/exp(-nfa_p/2*(pf*nfa/gdp)) ;

// 34. This is the home risk-free rate.
1/(1+rs) = beta_p *lambda(1)/lambda;

// 35. Consumption-based real exchange rate
q = pf*pc2/pc;

// 36. Capital
kpt = (1-delta_p)*kpt(-1) + jt + psik_p *(jt/kpt(-1)-delta_p)^2;
kpn = (1-delta_p)*kpn(-1) + jn + psik_p *(jn/kpn(-1)-delta_p)^2;

// 38. Rental Rate for Labor, FOC with respect to labor
wn = (1-alphan_p)*(yn/(len*nn))*nn;
wt = (1-alphat_p)*(yt/(let*nt))*nt;

// 40. Rental Rate for Capital
rn = alphan_p*(yn/kpn(-1));
rt = alphat_p*(yt/kpt(-1));

// 42. Labor-augmenting technology shock
nn-1 = rhonn_p*(nn(-1)-1) + nnerr;
nt-1 = rhont_p*(nt(-1)-1) + nterr;



///////////////////////////////////////////////////////////////////////////
// Country 2
///////////////////////////////////////////////////////////////////////////

// FOC with respect to consumption
1/cc2 - lambda2*pc2=0;   

// FOC wrt to aggregate labor
-chi0_p*(1-le2)^(-chi_p) + eta2 =0;

// FOC wrt to labor
lambda2*wn2*pn2-eta2 = 0;
lambda2*wt2-eta2 = 0;

// FOC with respect to investment
gamman2-lambda2*pj2 -gamman2*psik_p*(jn2/kpn2(-1)-delta_p)= 0;
gammat2-lambda2*pj2 -gammat2*psik_p*(jt2/kpt2(-1)-delta_p)= 0;


// gamman shadow price of capital
(1-delta_p)*beta_p * gamman2(1) - gamman2 + beta_p*lambda2(1)*pn2(1)*rn2(1) +beta_p*gamman2(1)*psik_p*(jn2(1)/kpn2-delta_p)/kpn2 = 0;
(1-delta_p)*beta_p * gammat2(1) - gammat2 + beta_p*lambda2(1)*rt2(1) +beta_p*gammat2(1)*psik_p*(jt2(1)/kpt2-delta_p)/kpt2 = 0;

// technology constraints

// non-traded output
yn2 = kpn2(-1)^alphan_p*(len2*nn2)^(1-alphan_p);
yt2 = kpt2(-1)^alphat_p*(let2*nt2)^(1-alphat_p);

// GDP
gdp2 = pn2*yn2+yt2;

// resource constraints
yn2 = ccn2 + in2;
yt2 = cch2 + ih2 + ccm+im;
j2 = jt2 + jn2;
le2 = len2 + let2;

// definition of consumption aggregator (pins down ccd)
cc2  = (omegacn_p^(rhon_p/(1+rhon_p))*ccn2^(1/(1+rhon_p))+(1-omegacn_p)^(rhon_p/(1+rhon_p))*cct2^(1/(1+rhon_p)))^(1+rhon_p);
cct2 = (omegact_p^(rhot_p/(1+rhot_p))*cch2^(1/(1+rhot_p))+(1-omegact_p)^(rhot_p/(1+rhot_p))*ccm2^(1/(1+rhot_p)))^(1+rhot_p);

// demand equations
ccn2 = omegacn_p*(pn2/pc2)^(-(1+rhon_p)/rhon_p) *cc2;
cct2 = (1-omegacn_p)*(pct2/pc2)^(-(1+rhon_p)/rhon_p) *cc2;
cch2 = omegact_p*(1/pct2)^(-(1+rhot_p)/rhot_p) *cct2;
ccm2 = (1-omegact_p)*(pf2/pct2)^(-(1+rhot_p)/rhot_p) *cct2;


// definition of consumption aggregator (pins down ccd)
j2 =  (omegain_p^(rhon_p/(1+rhon_p))*in2^(1/(1+rhon_p))+(1-omegain_p)^(rhon_p/(1+rhon_p))*it2^(1/(1+rhon_p)))^(1+rhon_p);
it2 = (omegait_p^(rhot_p/(1+rhot_p))*ih2^(1/(1+rhot_p))+(1-omegait_p)^(rhot_p/(1+rhot_p))*im2^(1/(1+rhot_p)))^(1+rhot_p);

// demand equations
in2 = omegain_p*(pn2/pj2)^(-(1+rhon_p)/rhon_p) *j2;
it2 = (1-omegain_p)*(pit2/pj2)^(-(1+rhon_p)/rhon_p) *j2;
ih2 = omegait_p*(1/pit2)^(-(1+rhot_p)/rhot_p) *it2;
im2 = (1-omegait_p)*(pf2/pit2)^(-(1+rhot_p)/rhot_p) *it2;

// Trade Balance, share of GDP
tby2 = (ccm+im-pf2*ccm2-pf2*im2)/gdp2;

// Saving rate
sy2 = 1-cc2/gdp2;

// Investment rate
jy2 = j2/gdp2;

pf2 = 1/pf;

// This is the home risk-free rate.
1/(1+rs2) = beta_p *lambda2(1)/lambda2;

// Consumption-based real exchange rate
q2 = pf2*pc/pc2;

// Capital
kpt2 = (1-delta_p)*kpt2(-1) + jt2;
kpn2 = (1-delta_p)*kpn2(-1) + jn2;

// Rental Rate for Labor, FOC with respect to labor
wn2 = (1-alphan_p)*(yn2/(len2*nn2))*nn2;
wt2 = (1-alphat_p)*(yt2/(let2*nt2))*nt2;

// Rental Rate for Capital
rn2 = alphan_p*(yn2/kpn2(-1));
rt2 = alphat_p*(yt2/kpt2(-1));

// Labor-augmenting technology shock
(nn2-1) = rhonn_p*(nn2(-1)-1) + nnerr2;
(nt2-1) = rhont_p*(nt2(-1)-1) + nterr2;

end;




initval;

///////////////////////////////////////////////////////////////////////////
// Steady state for  country 1
///////////////////////////////////////////////////////////////////////////

// net foreign assets
nfa =0;

// net foreign assets to gdp
nfay = 0;

tby = 0;

// consumption based-real exchange rate
q = 1;

// Rental rate for capital
rn = r_p;
rt = r_p;

// investment rate
jy = s_p;
sy = s_p;
j = s_p * y_p;
jn = sn_p * yn_p;
jt = st_p * yt_p;

in = omegain_p *j;
it = (1-omegain_p) *j;
ih = omegait_p * it;
im = (1-omegait_p) * it;

// short term real rate
rs = 1/beta_p -1;

// Labor
le = le_p;
let = let_p;
len = len_p;

// Output
gdp = y_p;
yn = yn_p;
yt = yt_p;

// Consumption
cc = gdp - j;
ccn = omegacn_p * cc;
cct = (1-omegacn_p) * cc;
cch = omegact_p * cct;
ccm = (1-omegact_p) * cct;

// relative prices
pf = 1;
pn = 1;
pc = 1;
pct = 1;
pj = 1;
pit = 1;

// Capital 
kpt = jt/delta_p;
kpn = jn/delta_p;

// Rental rate for labor
wn = (1-alphan_p)*yn_p/len_p;
wt = (1-alphat_p)*yt_p/let_p;

// Multipliers
lambda = 1/cc;

gamman = lambda;
gammat = lambda;

eta=chi0_p*(1-le_p)^(-chi_p);

// Labor-augmenting technology
nn=1;
nt=1;

///////////////////////////////////////////////////////////////////////////
// Steady state for country 2
///////////////////////////////////////////////////////////////////////////

// consumption based-real exchange rate
q2 = 1;

tby2 = 0;

// Rental rate for capital
rn2 = r_p;
rt2 = r_p;

// investment rate
jy2 = s_p;
sy2 = s_p;
j2 = s_p * y_p;
jn2 = sn_p * yn_p;
jt2 = st_p * yt_p;

in2 = omegain_p *j2;
it2 = (1-omegain_p) *j2;
ih2 = omegait_p * it2;
im2 = (1-omegait_p) * it2;

// short term real rate
rs2 = 1/beta_p -1;

// Labor
le2 = le_p;
let2 = let_p;
len2 = len_p;

// Output
gdp2 = y_p;
yn2 = yn_p;
yt2 = yt_p;

// Consumption
cc2 = gdp2 - j2;
ccn2 = omegacn_p * cc2;
cct2 = (1-omegacn_p) * cc2;
cch2 = omegact_p * cct2;
ccm2 = (1-omegact_p) * cct2;

// relative prices
pf2 = 1;
pn2 = 1;
pc2 = 1;
pct2 = 1;
pj2 = 1;
pit2 = 1;

// Capital 
kpt2 = jt2/delta_p;
kpn2 = jn2/delta_p;

// Rental rate for labor
wn2 = (1-alphan_p)*yn_p/len_p;
wt2 = (1-alphat_p)*yt_p/let_p;

// Multipliers
lambda2 = 1/cc2;

gamman2 = lambda2;
gammat2 = lambda2;

eta2 = chi0_p*(1-le_p)^(-chi_p);

// Labor-augmenting technology
nn2=1;
nt2=1;

end;

shocks;
var nnerr; stderr .01;
var nterr; stderr .001;
var nnerr2; stderr .1;
var nterr2; stderr .0001;
end;

check;


stoch_simul(order=1,nocorr,nomoments,irf=0);

//stoch_simul(order=1,nocorr,nomoments,noprint,irf=0);
